﻿<#########################################################################################################
 # File: Install-WindowsAdminCenterHA.ps1
 #
 # .DESCRIPTION
 #
 #  Install Windows Admin Center as HA service.
 #
 #  Copyright (c) Microsoft Corp 2017.
 #
 #########################################################################################################>

<#
.SYNOPSIS

Install Windows Admin Center as HA service.

.DESCRIPTION

The Install-WindowsAdminCenterHA.ps1 script installs sme on all nodes in failover cluster and create a generic serive role for it.

.PARAMETER clusterStorage
Specifies the path to the cluster shared volume.

.PARAMETER clientAccessPoint
Specifies the name for client access point.

.PARAMETER staticAddress
Specifies one or more static addresses for the cluster generic service.

.PARAMETER msiPath
Specifies the path of the Windows Admin Center msi installer.

.PARAMETER certPath
Specifies the path for ssl certificate.

.PARAMETER certPassword
Specifies the password for ssl certificate.

.PARAMETER portNumber
Specifies the ssl port number.

.PARAMETER generateSslCert
Generates a self signed ssl certificate.

.EXAMPLE

C:\PS> .\Install-WindowsAdminCenterHA.ps1 -clusterStorage C:\ClusterStorage\Volume1 -clientAccessPoint smeha-1 -msiPath '.\ServerManagementGateway.msi' -generateSslCert

.EXAMPLE
C:\PS> $certPassword = Read-Host -AsSecureString
C:\PS> .\Install-WindowsAdminCenterHA.ps1 -clusterStorage C:\ClusterStorage\Volume1 -clientAccessPoint smeha-1 -msiPath '.\ServerManagementGateway.msi' -certPath test.pfx -certPassword $certPassword
#>

[cmdletbinding(
    DefaultParameterSetName='existingCert'
)]
param (
    [Parameter(Mandatory = $true)]
    [String]
    $clusterStorage,
    [Parameter(Mandatory = $true)]
    [String]
    $clientAccessPoint,
    [String[]]
    $staticAddress,
    [String]
    $msiPath='.\ServerManagementGateway.msi',
    [Parameter(ParameterSetName='existingCert', Mandatory = $true)]
    [String]
    $certPath,
    [Parameter(ParameterSetName='existingCert', Mandatory = $true)]
    [SecureString]
    $certPassword,
    [int]
    $portNumber=443,
    [Parameter(ParameterSetName='newCert')]
    [switch]
    $generateSslCert
)

#Requires -RunAsAdministrator

Function Install {
    Param($certPassword, $certThumbprint, $certName, $portNumber, $tempFolder, $upgradeMode)

    $nodes = Get-ClusterNode
    foreach($node in $nodes)
    {
        Write-Output "installing Windows Admin Center on node: $node"
        $computer=$node.Name

        $command = { 
        param($certPassword, $certThumbprint, $certName, $portNumber, $tempFolder)
            Import-PfxCertificate -FilePath "$tempFolder\$certName" -Password $certPassword -CertStoreLocation Cert:\LocalMachine\My | Out-Null

            $filePath = "$tempFolder\ServerManagementGateway.msi"
            $logFile = "$tempFolder\sme-$env:computername.log"
            $processSInput = New-Object System.Diagnostics.ProcessStartInfo
            $processSInput.FileName = "$FilePath"
            $processSInput.Arguments = "/qn /l*v `"$logFile`" SME_PORT=$portNumber SSL_CERTIFICATE_OPTION=installed SME_THUMBPRINT=$certThumbprint SET_TRUSTED_HOSTS=`"*`""
            $process = [System.Diagnostics.Process]::Start($processSInput)
            #$process.ProcessName
            $process.WaitForExit()
            $exitCode = $process.ExitCode
            if($exitCode -ne 0)
            {
                Write-Error "installation failed, see details in log file $logFile"
            }

            #change service start mode to manual
            Set-Service ServerManagementGateway -startuptype "manual"
            Stop-Service ServerManagementGateway
        }

        Invoke-Command -ComputerName $computer -ScriptBlock $command -ArgumentList $certPassword, $certThumbprint, $certName, $portNumber, $tempFolder
    }
}

Function Get-TempPassword {
    $length=20
    $ascii=$NULL
    for ($a=33;$a –le 126;$a++) 
    {
        $ascii+=,[char][byte]$a 
    }

    for ($loop=1; $loop –le $length; $loop++) {
        $TempPassword+=($ascii | Get-Random)
    }

    return $TempPassword
}

$ErrorActionPreference = "Stop"

if (-Not (Test-Path -path $clusterStorage))
{
    Write-Error "Path not found: $clusterStorage"
}

$sharedVolumes = Get-ClusterSharedVolume
$isCsv = $false
foreach($volume in $sharedVolumes)
{
    if ($clusterStorage.StartsWith($volume.SharedVolumeInfo.FriendlyVolumeName, 1))
    {
        $isCsv = $true
        break
    }
}

if(-Not $isCsv)
{
    Write-Error "$clusterStorage is not cluster shared volume."
}

if (-Not $generateSslCert)
{
    $certPath = Resolve-Path $certPath
}

$msiPath = Resolve-Path $msiPath

if($generateSslCert)
{
    Write-Output "Creating self signed certificate"
    $domain= (Get-WmiObject win32_computersystem).Domain
    $dnsName = $clientAccessPoint+"."+$domain
    $cert = New-SelfSignedCertificate -DnsName $dnsName -CertStoreLocation "cert:\LocalMachine\My" -NotAfter (Get-Date).AddMonths(3)
    $tmpPassword = Get-Temppassword
    $certPassword = ConvertTo-SecureString -String $tmpPassword -Force -AsPlainText
    $certPath = "$PSScriptRoot\sme.pfx"
    $cert | Export-PfxCertificate -FilePath $certPath -Password $certPassword | Out-Null   
}
else
{
    $cert = New-Object System.Security.Cryptography.X509Certificates.X509Certificate2
    Write-Output $certPassword
    $cert.Import($certPath, $certPassword,'DefaultKeySet')
}

$certName = Split-Path $certPath -leaf
$certThumbprint = $cert.Thumbprint
$nodes = Get-ClusterNode


$tempFolder = "$clusterStorage\temp"
if (-Not (Test-Path -path $tempFolder)) {
    New-Item $tempFolder -Type Directory | Out-Null
}
Copy-Item -Path $certPath -Destination $tempFolder -Force
Copy-Item -Path $msiPath -Destination "$tempFolder\ServerManagementGateway.msi" -Force

Install $certPassword $certThumbprint $certName $portNumber $tempFolder

Write-Output "Adding Cluster Generic Service Role"
if ($staticAddress)
{
    $role = Add-ClusterGenericServiceRole -ServiceName ServerManagementGateway -Name $clientAccessPoint -CheckpointKey "SOFTWARE\Microsoft\ServerManagementGateway\Ha" -StaticAddress $staticAddress
}
else
{
    $role = Add-ClusterGenericServiceRole -ServiceName ServerManagementGateway -Name $clientAccessPoint -CheckpointKey "SOFTWARE\Microsoft\ServerManagementGateway\Ha"
}

$computer=$role.OwnerNode
$smePath = $clusterStorage+"\Server Management Experience"
$command = { 
    param (
    $smePath, $certThumbprint, $portNumber, $clientAccessPoint, $staticAddress
    )

    Write-Output "Copying files to cluster storage: $smePath"
    $UxFolder = "$smePath\Ux"
    if (Test-Path $UxFolder)
    {
        Remove-Item $UxFolder -Force -Recurse
    }

    New-Item -Path $UxFolder -ItemType directory | Out-Null

    Copy-Item -Path "$env:programdata\Server Management Experience\Ux"  -Destination $smePath -Recurse -Container -Force

    $registryPath = "HKLM:\Software\Microsoft\ServerManagementGateway\Ha"
    if (-Not (Test-Path $registryPath))
    {
        New-Item -Path $registryPath | Out-Null
    }
    New-ItemProperty -Path $registryPath -Name IsHaEnabled -Value "true" -PropertyType String -Force | Out-Null
    New-ItemProperty -Path $registryPath -Name StoragePath -Value $smePath -PropertyType String -Force | Out-Null
    New-ItemProperty -Path $registryPath -Name Thumbprint -Value $certThumbprint -PropertyType String -Force | Out-Null
    New-ItemProperty -Path $registryPath -Name Port -Value $portNumber -PropertyType DWord -Force | Out-Null
    New-ItemProperty -Path $registryPath -Name ClientAccessPoint -Value $clientAccessPoint -PropertyType String -Force | Out-Null
    $staticAddressValue = $staticAddress -join ','
    New-ItemProperty -Path $registryPath -Name StaticAddress -Value $staticAddress -PropertyType String -Force | Out-Null
    
    #grant permissions to Network Service for the UX folder
    $Acl = Get-Acl $UxFolder
	$sID = New-Object System.Security.Principal.SecurityIdentifier("S-1-5-20")
    $Ar = New-Object  system.security.accesscontrol.filesystemaccessrule($sID,"FullControl","ContainerInherit,ObjectInherit","None", "Allow")
    $Acl.SetAccessRule($Ar)
    Set-Acl $UxFolder $Acl

    Restart-Service ServerManagementGateway
}

Invoke-Command -ComputerName $computer -ScriptBlock $command -ArgumentList $smePath, $certThumbprint, $portNumber, $clientAccessPoint, $staticAddress
Write-Output "Installation is completed."