﻿<#########################################################################################################
 # File: Update-WindowsAdminCenterHA.ps1
 #
 # .DESCRIPTION
 #
 #  Update Windows Admin Center Ha service.
 #
 #  Copyright (c) Microsoft Corp 2017.
 #
 #########################################################################################################>

 <#
.SYNOPSIS

Install Windows Admin Center as HA service.

.DESCRIPTION

The Update-WindowsAdminCenterHA.ps1 script updates sme on all nodes in failover cluster.

.PARAMETER msiPath
Specifies the path of the Windows Admin Center msi installer.

.EXAMPLE

C:\PS> .\Update-WindowsAdminCenterHA.ps1 -msiPath '.\ServerManagementGateway.msi'
#>

param (
    [String]
    $msiPath='.\ServerManagementGateway.msi'
)

#Requires -RunAsAdministrator

Function Install {
    Param($certThumbprint, $portNumber, $tempFolder)

    $nodes = Get-ClusterNode
    foreach($node in $nodes)
    {
        Write-Output "installing Windows Admin Center on node: $node"
        $computer=$node.Name

        $command = { 
        param($certThumbprint, $portNumber, $tempFolder)
            $filePath = "$tempFolder\ServerManagementGateway.msi"
            $logFile = "$tempFolder\sme-$env:computername.log"
            $processSInput = New-Object System.Diagnostics.ProcessStartInfo
            $processSInput.FileName = "$FilePath"
            $processSInput.Arguments = "/qn /l*v `"$logFile`" SME_PORT=$portNumber SSL_CERTIFICATE_OPTION=installed SME_THUMBPRINT=$certThumbprint SET_TRUSTED_HOSTS=`"*`""
            $process = [System.Diagnostics.Process]::Start($processSInput)
            #$process.ProcessName
            $process.WaitForExit()
            $exitCode = $process.ExitCode
            if($exitCode -ne 0)
            {
                Write-Error "installation failed, see details in log file $logFile"
            }

            #change service start mode to manual
            Set-Service ServerManagementGateway -startuptype "manual"
            Stop-Service ServerManagementGateway
        }

        Invoke-Command -ComputerName $computer -ScriptBlock $command -ArgumentList $certThumbprint, $portNumber, $tempFolder
    }
}

$ErrorActionPreference = "Stop"

$resource = Get-ClusterResource | where {$_.Name -like '*Windows Admin Center*'}
if(-Not $resource)
{
    write-error 'Windows Admin Center is not installed.'
}

$ownerGroup = $resource.OwnerGroup
$ownerNode = $ownerGroup.OwnerNode

$command = { 
    Get-ItemProperty "HKLM:\Software\Microsoft\ServerManagementGateway\ha"
}

$result = Invoke-Command -ComputerName $ownerNode -ScriptBlock $command
$storagePath = $result.StoragePath
$certThumbprint = $result.Thumbprint
$portNumber = $result.Port
$clientAccessPoint = $result.clientAccessPoint
if($result.staticAddress)
{
    $staticAddress = $result.staticAddress.Split(',')
}

$msiPath = Resolve-Path $msiPath

$clusterStorage = (get-item $storagePath).parent.fullname
$tempFolder = "$clusterStorage\temp"
if (-Not (Test-Path -path $tempFolder)) {
    New-Item $tempFolder -Type Directory
}

Copy-Item -Path $msiPath -Destination "$tempFolder\ServerManagementGateway.msi" -Force

Remove-ClusterGroup $ownerGroup.Name -RemoveResources -Force

Install $certThumbprint $portNumber $tempFolder

Write-Output "Adding Cluster Generic Service Role"
if ($staticAddress)
{
    $role = Add-ClusterGenericServiceRole -ServiceName ServerManagementGateway -Name $clientAccessPoint -CheckpointKey "SOFTWARE\Microsoft\ServerManagementGateway\Ha" -StaticAddress $staticAddress
}
else
{
    $role = Add-ClusterGenericServiceRole -ServiceName ServerManagementGateway -Name $clientAccessPoint -CheckpointKey "SOFTWARE\Microsoft\ServerManagementGateway\Ha"
}

$computer=$role.OwnerNode

$command = { 
    param (
    $smePath
    )
    Write-Output "Copying files to cluster storage: $smePath"
    
    $UxFolder = "$smePath\Ux"
    if (Test-Path $UxFolder)
    {
        Remove-Item $UxFolder -Force -Recurse
    }

    New-Item -Path $UxFolder -ItemType directory | Out-Null
    Copy-Item -Path "$env:programdata\Server Management Experience\Ux"  -Destination $smePath -Recurse -Container -Force

    #grant permissions to Network Service for the UX folder
    $Acl = Get-Acl $UxFolder
	$sID = New-Object System.Security.Principal.SecurityIdentifier("S-1-5-20")
    $Ar = New-Object  system.security.accesscontrol.filesystemaccessrule($sID,"FullControl","ContainerInherit,ObjectInherit","None", "Allow")
    $Acl.SetAccessRule($Ar)
    Set-Acl $UxFolder $Acl

    Restart-Service ServerManagementGateway
}

Invoke-Command -ComputerName $computer -ScriptBlock $command -ArgumentList $storagePath

Write-Output "Installation is completed."